#include <iostream>
#include <fstream>
#include <ctime>
#include <random>
#include <stdlib.h>
#include <string>


#include <igl/readOFF.h>
#include <igl/writeOBJ.h>


#include "mesh_generation/CSG_graph.hpp"


int main(int argc, char *argv[])
{
    if (argc < 2)
    {
        std::cout<<"usage : "<<argv[0]<<" path batch_size id_batch\n Generate escad files and off meshes in folders path/escad and path/off. Create batch_size objects for batch number id_batch (start at batch_size*id_batch)\n";
        return 1;
    }

    std::string path = argv[1];
    int batch_size = std::stoi(argv[2]);
    int batch_id = std::stoi(argv[3]);

    //create directories
    std::string mkdir = "mkdir -p "+path+"/{escad,off}";
    system(mkdir.c_str());

    std::default_random_engine generator(time(NULL)+(1000*batch_id));
    std::uniform_int_distribution<int> nb_prim_distr(1,6); //between 1 and 6 primitives
    double max_size = 4.0;
    std::cout<<"===================================batch #"<<batch_id<<" of size "<<batch_size<<std::endl;
    for(int run = batch_size*batch_id; run < batch_size*(batch_id+1); run ++)
    {
        std::cout<<"===========object #"<<run<<std::endl;
        int nb_prim = nb_prim_distr(generator);
        //generate graph and save in escad file
        std::shared_ptr<CSG_node> current_graph =
                CSG_node::generate_CSG_tree(generator, nb_prim, max_size, "tmp"+std::to_string(batch_id));
        std::ofstream file(path+"/escad/"+std::to_string(run)+".escad");
        file<<*current_graph;
        file.close();

        //generate off file
        std::string cmd = OPENSCAD_PATH" -o "+path+"/off/"+std::to_string(run)+".off -D '$fa=6' -D '$fs=0.1' "+path+"/escad/"+std::to_string(run)+".escad";
        system(cmd.c_str());
    }

}
