//
// Created by delanoy on 2/9/18.
//

#ifndef ESCAD_PROJECT_CSG_GRAPH_HPP
#define ESCAD_PROJECT_CSG_GRAPH_HPP


#include <string>
#include <iostream>
#include <sstream>
#include <memory>
#include <list>
#include <random>

#include "primitive.hpp"
#include <stack>
#include <string>
#include <fstream>

//set according to your configuration
#define OPENSCAD_PATH "openscad"


class CSG_node
{
public:
    typedef enum {UNION, DIFF} CSG_operator;
    //return the list of primitives of the graph
    virtual std::list<std::shared_ptr<Primitive> > getPrimitives()=0;
    //read graph from a file
    static std::shared_ptr<CSG_node> read_graph(std::string filename);
    //generate a tree composed of nb_prim prinitives. temporary objects will eb stored in tmp_file
    static std::shared_ptr<CSG_node>generate_CSG_tree(std::default_random_engine &generator,
                                                      int nb_prim, double max_size, std::string tmp_file);

private:
    virtual std::ostream& printScript(std::ostream& str) const = 0;
    //write the graph in Openscad format
    friend std::ostream& operator<<(std::ostream& str, const CSG_node& graph)
    {
        return graph.printScript(str);
    }
};

class CSG_graph :  public  CSG_node //a subgraph composed of a CSG operation and 2 childs
{
private:
    CSG_operator _op; //diff or union
    // 2 childs
    std::shared_ptr<CSG_node>  _leftSide;
    std::shared_ptr<CSG_node>  _rightSide;
public:
    CSG_graph(CSG_operator op,std::shared_ptr<CSG_node>  leftSide,std::shared_ptr<CSG_node>  rightSide) :
            _op(op), _leftSide(leftSide), _rightSide(rightSide) {}

    std::list<std::shared_ptr<Primitive> > getPrimitives()
    {
        std::list<std::shared_ptr<Primitive> > Lp = _leftSide->getPrimitives();
        std::list<std::shared_ptr<Primitive> > Rp = _rightSide->getPrimitives();
        Rp.splice(Rp.begin(), Lp);
        return Rp;
    }

    std::ostream& printScript(std::ostream& str) const
    {
        if (_op == CSG_operator::UNION)
            str << "union() { \n";
        else if (_op == CSG_operator::DIFF)
            str << "difference() { \n";
        str << *_leftSide << *_rightSide;
        str << "} \n";
        return str;
    }
    friend std::ostream& operator<<(std::ostream& str, const CSG_graph& graph)
    {
        return graph.printScript(str);
    }
};

class CSG_leave :  public CSG_node //a leaf of the graph : contains only a primitive
{
private :
    std::shared_ptr<Primitive> _p;
public:
    CSG_leave(std::shared_ptr<Primitive> p) :
            _p(p) {}

    std::list<std::shared_ptr<Primitive> > getPrimitives()
    {
        std::list<std::shared_ptr<Primitive> > l;
        l.push_back(_p);
        return l;
    }

    std::ostream& printScript(std::ostream& str) const
    {
        return str << *_p;
    }
    friend std::ostream& operator<<(std::ostream& str, const CSG_leave& node)
    {
        return node.printScript(str);
    }
};


#endif //ESCAD_PROJECT_CSG_GRAPH_HPP
